import logging

class prompt(object):
    """A base class for a very simple command line interface"""
    def __init__(self):
        self.commands = {}
        self.prompt_text = '> '

    def start(self):
        """Starts the prompt"""
        self.register_command('help', 'Display a list of commands', self.help)

        self.start_prompt()

    def start_prompt(self):
        """Prompts the user for commands"""
        command = ''
        while True:
            command_args = raw_input(self.prompt_text).split()
            if not command_args:
                continue  # No input.
            elif len(command_args) == 1:
                command = command_args[0]
                args = []
            elif len(command_args) >= 2:
                command = command_args[0]
                args = command_args[1:]

            if command == 'exit':
                break

            if command in self.commands.keys():
                accepted_args = self.commands[command][2]
                if not args:  # If no arguments were provided...
                    if accepted_args == 0:  # Perfect!
                        self.commands[command][1]()
                    else:
                        print _('Not enough arguments provided')
                        continue
                else:  # Arguments were provided.
                    if accepted_args == 0:  # We didn't want args!
                        print _('%s does not accept any arguments' % command)
                        continue
                    elif accepted_args >= 1:  # Good. We wanted args.
                        if len(args) == accepted_args:  # Correct number of args
                            self.commands[command][1](args)
                        else:  # Wrong number of args
                            print _('%s takes %s argument(s)' % \
                                    (command, accepted_args))
                            continue
                    elif accepted_args == -1:  # Any number of args is good!
                            self.commands[command][1](args)                        
            else:
                print _('Command not found: %s' % command)
                    
    def register_command(self, command, help, function, args=0):
        """Add a command, help message, and function to self.commands"""
        # Note about args: 
        # * -1 is unlimited args, 
        # * 0 is no args, 
        # * 1 through inf. will specify a precise number
        if not self.commands.has_key(command):
            self.commands.update({command: (help, function, args)})
        else:
            logging.error(_('Unable to register command %s. It is already ' \
                        'registered.'))

    def remove_command(self, command):
        """Remove a command"""
        if not self.commands.has_key(command):
            logging.error(_('Unable to remove command %s. It is not ' \
                        'registered.'))
        else:
            del(self.commands[command])

    ### Commands ###        

    def help(self):
        """Display a list of commands"""
        # Get the command names.
        command_names = self.commands.keys()

        # Get a string of spaces to separate commands from help strings.
        space = ' ' * len(longest_item(command_names))

        # Start printing the commands
        for command in sorted(command_names):
            help = self.commands[command][0]
            print '%s%s  %s' % (command, space[len(command):], help)

def longest_item(a_list):
    """Returns the longest item in a list/tuple"""
    # Make sure we have a list or tuple.
    if not type(a_list) in (list, tuple):
        return
    
    # Get the lingest item.
    big = ''
    for item in a_list:
        if len(item) > len(big):
            big = item
    return big
