#!/usr/bin/python -tt
# -*- coding: UTF-8 -*-
# vim: sw=4 ts=4 expandtab ai
#
# Copyright (C) 2010 Chris Oliver <excid3@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301 USA
 

import bz2
import gzip
import os
import sys
import urllib
 
 
def url_to_filename(url):
    """Convert a file url to a filename
       Example: use this function to take the Packages url to a filename
    """
    return url[7:].replace("/","_")


def url_join(args):
    """Joins a list of arguments by /'s to construct a url"""
    return "/".join([x.rstrip("/") for x in args])


# Do we really need to override this for 206? I haven't had any issue
#class Opener(urllib.FancyURLopener):
#    def __init__(self, proxies):
#        urllib.FancyURLOpener(proxies)
#    def http_error_206(self, url, fp, errcode, errmsg, headers, data=None):
#        pass


def text_progress(url, progress, total):
    """Reports the progress of a transfer inline in the terminal"""
    sys.stdout.write("\r%s ... %2d%%" % (url, (float(progress) / total * 100)))
    sys.stdout.flush()


def download(url, filename=None, proxies={}, progress=text_progress, display_text=None):
    """Downloads a file over HTTP with resume and progress support
       Example Usage:
       download("http://excid3.com/files/ubuntu-910.png",
                "ubuntu.png",
                {"http": "http:192.168.0.103:8888"},
                text_progress, # This is a function, see above for an example
                display_text="A customizable filename")
    """
    # Construct the filename from the url if a filename isn't provided
    if not filename:
        filename = url.split('/')[-1]
    
    # Expand
    filename = os.path.expanduser(os.path.expandvars(filename))

    if not display_text:
        display_text = url

    url_opener = urllib.FancyURLopener(proxies)
    page = url_opener.open(url)

    # If it's not an OK, we can't download the file
    if page.getcode() != 200:
        raise urllib.error.URLError, "Invalid url: %s" % url

    if not page.headers.has_key('Content-Length'):
        page.headers['Content-Length'] = "0"

    # Determine if we have a partial download of this file ready
    existing = 0
    length = int(page.headers['Content-Length'])
    if os.path.exists(filename):
        outfile = open(filename, "ab")
        existing = os.path.getsize(filename)
        url_opener.addheader("Range", "bytes=%s-" % existing)
    else:
        outfile = open(filename, "wb")

    # It's already fully downloaded
    if length == existing:
        return

    total = existing + length
    downloaded = 0

    while True:
        data = page.read(8192)

        if not data:
            break

        if progress:
            progress(display_text, downloaded + existing, total)

        outfile.write(data)
        downloaded += len(data)

    if progress:
        progress(display_text, total, total)
    #FIXME: move this line to progress function
    sys.stdout.write("\r%s ... 100%%\n" % display_text)

    page.close()
    outfile.close()


def decompress(filename, remove=True):
    """Decompress any bz2 or gz file"""
    if filename.endswith(".bz2"):
        target = filename[:-4]
        data = bz2.BZ2File(filename).read()

    elif filename.endswith(".gz"):
        target = filename[:-3]
        data = gzip.GzipFile(filename).read()

    else:
        raise AttributeError, "Invalid compression type: %s" % filename

    # Open the decompressed file, write the data into it
    f = open(target, "wb")
    f.write(data)
    f.close()

    # Remove the compressed version
    if remove:
        os.remove(filename)

    # Return the decompressed filename
    return target
