import os
import sys
import urllib

class download(object):
    """Downloads a file over HTTP with resume and progress support
    Example Usage:
    download("http://excid3.com/files/ubuntu-910.png",
    "ubuntu.png",
    {"http": "http:192.168.0.103:8888"},
    text_progress, # This is a function, see above for an example
    display_text="A customizable filename")
    """

    def __init__(self, url, filename=None, proxies={}, progress=text_progress,
                 display_text=None):
        # Set URL.
        self.url = url
        # Set filename.
        self.filename = filename
        if not self.filename:
            self.filename = url.split('/')[-1]
        self.filename = os.path.expandvars(os.path.expanduser(self.filename))
        # Set proxies.
        self.proxies = proxies
        # Set display_text.
        if display_text:
            self.display_text = display_text
        else:
            self.display_text = url

        # Can't stop before we even get started.
        self.going = True

        # Aliases:
        self.start = self.download
        self.resume = self.download

    def download(self):
        """Downloads the file"""
        existing = 0
        url_opener = urllib.FancyURLopener(proxies)
     
        if os.path.exists(filename):
            outfile = open(filename, "ab")
            existing = os.path.getsize(filename)
            url_opener.addheader("Range", "bytes=%s-" % existing)
        else:
            outfile = open(filename, "wb")
     
        page = url_opener.open(url)
        if not page.headers.has_key('Content-Length'):
            page.headers['Content-Length'] = "0"
     
        length = int(page.headers['Content-Length'])
        if length == existing:
            return  # Everything has already been downloaded.
     
        total = existing + length
        downloaded = 0
     
        while self.going:
            data = page.read(8192)
     
            if not data:
                break
     
            if progress:
                progress(display_text, downloaded + existing, total)
     
            outfile.write(data)
            downloaded += len(data)
     
        sys.stdout.write("\r%s ... 100%%\n" % display_text)
        page.close()
        outfile.close()

    def stop(self):
        """Stops the download"""
        self.going = False


def text_progress(url, progress, total):
    """Reports the progress of a transfer inline in the terminal"""
    sys.stdout.write("\r%s ... %2d%%" % (url, (float(progress) / total * 100)))
    sys.stdout.flush()


def url_to_filename(url):
    """Convert a file url to a filename
    Example: use this function to take the Packages url to a filename
    """
    return url[7:].replace("/","_")
 
 
def url_join(args):
    """Joins a list of arguments by /'s to construct a url"""
    return "/".join([x.rstrip("/") for x in args])
